#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for ibmdbpy.learn.assocation_rules
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

class Test_AssociationRulesInitiateModel(object):

    def test_arules_instance(self, idadf):
        pass

    def test_arules_get_parameters(self, idadf):
        pass

    def test_arules_set_parameters(self, idadf):
        pass

class Test_AssociationRulesFitandPredict(object):

    def test_arules_fit(self, idadf):
        pass

    def test_arules_predict(self, idadf):
        pass

    def test_arules_fit_and_predict(self, idadf):
        pass

class Test_AssociationRulesExploreResult(object):

    def test_arules_describe(self, idadf):
        pass

    def test_arules_get_labels(self, idadf):
        pass

    def test_arules_retrieve_KMeans_Model(self, idadf):
        pass